@@ -18,7 +18,6 @@
 # - ``SERVICE_{TENANT_NAME|PASSWORD}`` must be defined
 # - ``SERVICE_HOST``
 # - ``OS_AUTH_URL``, ``KEYSTONE_SERVICE_URI`` for auth in api
-# - ``CEILOMETER_CONF`` for ceilometer dispatcher configuration
 
 # stack.sh
 # ---------
@@ -61,15 +60,14 @@ function create_gnocchi_accounts {
     then
         create_service_user "gnocchi"
 
-        if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
-            local gnocchi_service=$(get_or_create_service "gnocchi" \
-                "metric" "OpenStack Metric Service")
-            get_or_create_endpoint $gnocchi_service \
-                "$REGION_NAME" \
-                "$(gnocchi_service_url)/" \
-                "$(gnocchi_service_url)/" \
-                "$(gnocchi_service_url)/"
-        fi
+        local gnocchi_service=$(get_or_create_service "gnocchi" \
+            "metric" "OpenStack Metric Service")
+        get_or_create_endpoint $gnocchi_service \
+            "$REGION_NAME" \
+            "$(gnocchi_service_url)" \
+            "$(gnocchi_service_url)" \
+            "$(gnocchi_service_url)"
+
         if is_service_enabled swift && [[ "$GNOCCHI_STORAGE_BACKEND" = 'swift' ]] ; then
             get_or_create_project "gnocchi_swift" default
             local gnocchi_swift_user=$(get_or_create_user "gnocchi_swift" \
@@ -228,16 +226,20 @@ function configure_gnocchi {
     # Install the configuration files
     cp $GNOCCHI_DIR/etc/gnocchi/* $GNOCCHI_CONF_DIR
 
-    iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
-    if [ "${GNOCCHI_COORDINATOR_URL:0:7}" == "file://" ]; then
-        gnocchi_locks_dir=${GNOCCHI_COORDINATOR_URL:7}
-        [ ! -d $gnocchi_locks_dir ] && sudo mkdir -m 755 -p ${gnocchi_locks_dir}
-        sudo chown $STACK_USER $gnocchi_locks_dir
+
+    if [ -n "$GNOCCHI_COORDINATOR_URL" ]; then
+        iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
     fi
 
     # Configure auth token middleware
     configure_auth_token_middleware $GNOCCHI_CONF gnocchi $GNOCCHI_AUTH_CACHE_DIR
 
+    if is_service_enabled gnocchi-statsd ; then
+        iniset $GNOCCHI_CONF statsd resource_id $GNOCCHI_STATSD_RESOURCE_ID
+        iniset $GNOCCHI_CONF statsd project_id $GNOCCHI_STATSD_PROJECT_ID
+        iniset $GNOCCHI_CONF statsd user_id $GNOCCHI_STATSD_USER_ID
+    fi
+
     # Configure the storage driver
     if is_service_enabled ceph && [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
         iniset $GNOCCHI_CONF storage driver ceph
@@ -263,14 +265,16 @@ function configure_gnocchi {
 
     if is_service_enabled key; then
         if is_service_enabled gnocchi-grafana; then
-            iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline "cors keystone_authtoken gnocchi"
+            iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline "cors gnocchi+auth"
             iniset $KEYSTONE_CONF cors allowed_origin ${GRAFANA_URL}
             iniset $GNOCCHI_CONF cors allowed_origin ${GRAFANA_URL}
             iniset $GNOCCHI_CONF cors allow_methods GET,POST,PUT,DELETE,OPTIONS,HEAD,PATCH
             iniset $GNOCCHI_CONF cors allow_headers Content-Type,Cache-Control,Content-Language,Expires,Last-Modified,Pragma,X-Auth-Token,X-Subject-Token
+        else
+            iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi+auth
         fi
     else
-        iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi
+        iniset $GNOCCHI_PASTE_CONF pipeline:main pipeline gnocchi+noauth
     fi
 
     # Configure the indexer database
@@ -294,25 +298,6 @@ function configure_ceph_gnocchi {
     sudo chown ${STACK_USER}:$(id -g -n $whoami) ${CEPH_CONF_DIR}/ceph.client.${GNOCCHI_CEPH_USER}.keyring
 }
 
-function configure_ceilometer_gnocchi {
-    gnocchi_url=$(gnocchi_service_url)
-    iniset $CEILOMETER_CONF DEFAULT dispatcher gnocchi
-    iniset $CEILOMETER_CONF alarms gnocchi_url $gnocchi_url
-    iniset $CEILOMETER_CONF dispatcher_gnocchi url $gnocchi_url
-    iniset $CEILOMETER_CONF dispatcher_gnocchi archive_policy ${GNOCCHI_ARCHIVE_POLICY}
-    if is_service_enabled swift && [[ "$GNOCCHI_STORAGE_BACKEND" = 'swift' ]] ; then
-        iniset $CEILOMETER_CONF dispatcher_gnocchi filter_service_activity "True"
-        iniset $CEILOMETER_CONF dispatcher_gnocchi filter_project "gnocchi_swift"
-    else
-        iniset $CEILOMETER_CONF dispatcher_gnocchi filter_service_activity "False"
-    fi
-}
-
-function configure_aodh_gnocchi {
-    gnocchi_url=$(gnocchi_service_url)
-    iniset $AODH_CONF DEFAULT gnocchi_url $gnocchi_url
-}
-
 
 # init_gnocchi() - Initialize etc.
 function init_gnocchi {
@@ -323,8 +308,8 @@ function init_gnocchi {
 
     if is_service_enabled mysql postgresql; then
         recreate_database gnocchi
-        $GNOCCHI_BIN_DIR/gnocchi-dbsync
     fi
+    $GNOCCHI_BIN_DIR/gnocchi-upgrade
 }
 
 function preinstall_gnocchi {
@@ -334,6 +319,10 @@ function preinstall_gnocchi {
     else
         install_package postgresql-devel
     fi
+    if [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
+            install_package cython
+            install_package librados-dev
+    fi
 }
 
 # install_gnocchi() - Collect source and prepare
@@ -347,6 +336,10 @@ function install_gnocchi {
         pip_install influxdb
     fi
 
+    if [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
+        pip_install cradox
+    fi
+
     if is_service_enabled gnocchi-grafana
     then
         _gnocchi_install_grafana
@@ -354,8 +347,10 @@ function install_gnocchi {
 
     install_gnocchiclient
 
+    is_service_enabled key && EXTRA_FLAVOR=,keystonmiddleware
+
     # We don't use setup_package because we don't follow openstack/requirements
-    sudo -H pip install -e "$GNOCCHI_DIR"
+    sudo -H pip install -e "$GNOCCHI_DIR"[test,$GNOCCHI_STORAGE_BACKEND,${DATABASE_TYPE}${EXTRA_FLAVOR}]
 
     if [ "$GNOCCHI_USE_MOD_WSGI" == "True" ]; then
         install_apache_wsgi
@@ -396,22 +391,16 @@ function start_gnocchi {
     fi
 
     # Create a default policy
-    archive_policy_url="$(gnocchi_service_url)/v1/archive_policy"
     if ! is_service_enabled key; then
         export OS_AUTH_TYPE=gnocchi-noauth
         export GNOCCHI_USER_ID=`uuidgen`
         export GNOCCHI_PROJECT_ID=`uuidgen`
-        export GNOCCHI_ENDPOINT="${gnocchi_service_url}"
+        export GNOCCHI_ENDPOINT="$(gnocchi_service_url)"
     fi
 
-    gnocchi archive-policy create -d granularity:5m,points:12 -d granularity:1h,points:24 -d granularity:1d,points:30 low
-    gnocchi archive-policy create -d granularity:60s,points:60 -d granularity:1h,points:168 -d granularity:1d,points:365 medium
-    gnocchi archive-policy create -d granularity:1s,points:86400 -d granularity:1m,points:43200 -d granularity:1h,points:8760 high
-
-    gnocchi archive-policy-rule create -a low -m "*" default
-
     # run metricd last so we are properly waiting for swift and friends
     run_process gnocchi-metricd "$GNOCCHI_BIN_DIR/gnocchi-metricd -d -v --config-file $GNOCCHI_CONF"
+    run_process gnocchi-statsd "$GNOCCHI_BIN_DIR/gnocchi-statsd -d -v --config-file $GNOCCHI_CONF"
 }
 
 # stop_gnocchi() - Stop running processes
@@ -445,14 +434,6 @@ if is_service_enabled gnocchi-api; then
         echo_summary "Configuring Gnocchi"
         configure_gnocchi
         create_gnocchi_accounts
-        if is_service_enabled ceilometer; then
-            echo_summary "Configuring Ceilometer for gnocchi"
-            configure_ceilometer_gnocchi
-        fi
-        if is_service_enabled aodh; then
-            echo_summary "Configuring Aodh for gnocchi"
-            configure_aodh_gnocchi
-        fi
         if is_service_enabled ceph && [[ "$GNOCCHI_STORAGE_BACKEND" = 'ceph' ]] ; then
             echo_summary "Configuring Gnocchi for Ceph"
             configure_ceph_gnocchi
