@@ -62,8 +62,8 @@ class TestIndexerDriver(tests_base.TestCase):
                           self.index.delete_archive_policy,
                           str(uuid.uuid4()))
         metric_id = uuid.uuid4()
-        self.index.create_metric(metric_id, uuid.uuid4(),
-                                 uuid.uuid4(), "low")
+        self.index.create_metric(metric_id, str(uuid.uuid4()),
+                                 str(uuid.uuid4()), "low")
         self.assertRaises(indexer.ArchivePolicyInUse,
                           self.index.delete_archive_policy,
                           "low")
@@ -84,21 +84,21 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_create_metric(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         m = self.index.create_metric(r1, user, project, "low")
         self.assertEqual(r1, m.id)
         self.assertEqual(m.created_by_user_id, user)
         self.assertEqual(m.created_by_project_id, project)
         self.assertIsNone(m.name)
         self.assertIsNone(m.resource_id)
-        m2 = self.index.get_metrics([r1])
+        m2 = self.index.list_metrics(id=r1)
         self.assertEqual([m], m2)
 
     def test_expunge_metric(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         m = self.index.create_metric(r1, user, project, "low")
         self.index.delete_metric(m.id)
         try:
@@ -117,8 +117,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_create_resource(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         rc = self.index.create_resource('generic', r1, user, project)
         self.assertIsNotNone(rc.started_at)
         self.assertIsNotNone(rc.revision_start)
@@ -131,6 +131,7 @@ class TestIndexerDriver(tests_base.TestCase):
                           "project_id": None,
                           "started_at": rc.started_at,
                           "ended_at": None,
+                          "original_resource_id": None,
                           "type": "generic",
                           "metrics": {}},
                          rc.jsonify())
@@ -142,7 +143,7 @@ class TestIndexerDriver(tests_base.TestCase):
         e = uuid.uuid4()
         try:
             self.index.create_resource(
-                'generic', uuid.uuid4(), uuid.uuid4(), uuid.uuid4(),
+                'generic', uuid.uuid4(), str(uuid.uuid4()), str(uuid.uuid4()),
                 metrics={"foo": e})
         except indexer.NoSuchMetric as ex:
             self.assertEqual(e, ex.metric)
@@ -151,8 +152,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_create_resource_already_exists(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_resource('generic', r1, user, project)
         self.assertRaises(indexer.ResourceAlreadyExists,
                           self.index.create_resource,
@@ -160,24 +161,24 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_create_resource_with_new_metrics(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         rc = self.index.create_resource(
             'generic', r1, user, project,
             metrics={"foobar": {"archive_policy_name": "low"}})
         self.assertEqual(1, len(rc.metrics))
-        m = self.index.get_metrics([rc.metrics[0].id])
+        m = self.index.list_metrics(id=rc.metrics[0].id)
         self.assertEqual(m[0], rc.metrics[0])
 
-    def _do_test_create_instance(self, server_group=None):
+    def _do_test_create_instance(self, server_group=None, image_ref=None):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         kwargs = {'server_group': server_group} if server_group else {}
 
         rc = self.index.create_resource('instance', r1, user, project,
                                         flavor_id="1",
-                                        image_ref="http://foo/bar",
+                                        image_ref=image_ref,
                                         host="foo",
                                         display_name="lol", **kwargs)
         self.assertIsNotNone(rc.started_at)
@@ -195,8 +196,9 @@ class TestIndexerDriver(tests_base.TestCase):
                           "display_name": "lol",
                           "server_group": server_group,
                           "host": "foo",
-                          "image_ref": "http://foo/bar",
+                          "image_ref": image_ref,
                           "flavor_id": "1",
+                          "original_resource_id": None,
                           "metrics": {}},
                          rc.jsonify())
         rg = self.index.get_resource('generic', r1, with_metrics=True)
@@ -205,22 +207,27 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertEqual(rc.metrics, rg.metrics)
 
     def test_create_instance(self):
-        self._do_test_create_instance()
+        self._do_test_create_instance(image_ref='http://foo/bar')
 
     def test_create_instance_with_server_group(self):
-        self._do_test_create_instance('my_autoscaling_group')
+        self._do_test_create_instance('my_autoscaling_group',
+                                      image_ref='http://foo/bar')
+
+    def test_create_instance_without_image_ref(self):
+        self._do_test_create_instance(image_ref=None)
 
     def test_delete_resource(self):
         r1 = uuid.uuid4()
-        self.index.create_resource('generic', r1, uuid.uuid4(), uuid.uuid4())
+        self.index.create_resource('generic', r1, str(uuid.uuid4()),
+                                   str(uuid.uuid4()))
         self.index.delete_resource(r1)
         self.assertRaises(indexer.NoSuchResource,
                           self.index.delete_resource,
                           r1)
 
     def test_delete_resource_with_metrics(self):
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
         self.index.create_metric(e1, user, project,
@@ -234,7 +241,7 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertRaises(indexer.NoSuchResource,
                           self.index.delete_resource,
                           r1)
-        metrics = self.index.get_metrics([e1, e2])
+        metrics = self.index.list_metrics(ids=[e1, e2])
         self.assertEqual([], metrics)
 
     def test_delete_resource_non_existent(self):
@@ -246,8 +253,8 @@ class TestIndexerDriver(tests_base.TestCase):
     def test_create_resource_with_start_timestamp(self):
         r1 = uuid.uuid4()
         ts = utils.datetime_utc(2014, 1, 1, 23, 34, 23, 1234)
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         rc = self.index.create_resource(
             'generic',
             r1, user, project,
@@ -261,6 +268,7 @@ class TestIndexerDriver(tests_base.TestCase):
                           "project_id": None,
                           "started_at": ts,
                           "ended_at": None,
+                          "original_resource_id": None,
                           "type": "generic",
                           "metrics": {}}, rc.jsonify())
         r = self.index.get_resource('generic', r1, with_metrics=True)
@@ -270,8 +278,8 @@ class TestIndexerDriver(tests_base.TestCase):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1,
                                  user, project,
                                  archive_policy_name="low")
@@ -291,6 +299,7 @@ class TestIndexerDriver(tests_base.TestCase):
                           "project_id": None,
                           "started_at": rc.started_at,
                           "ended_at": None,
+                          "original_resource_id": None,
                           "type": "generic",
                           "metrics": {'foo': str(e1), 'bar': str(e2)}},
                          rc.jsonify())
@@ -306,6 +315,7 @@ class TestIndexerDriver(tests_base.TestCase):
                           "ended_at": None,
                           "user_id": None,
                           "project_id": None,
+                          "original_resource_id": None,
                           "metrics": {'foo': str(e1), 'bar': str(e2)}},
                          r.jsonify())
 
@@ -320,8 +330,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_update_resource_end_timestamp(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_resource('generic', r1, user, project)
         self.index.update_resource(
             'generic',
@@ -356,14 +366,15 @@ class TestIndexerDriver(tests_base.TestCase):
                           "project_id": None,
                           "type": "generic",
                           "started_at": r.started_at,
+                          "original_resource_id": None,
                           "metrics": {}}, r.jsonify())
 
     def test_update_resource_metrics(self):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1, user, project,
                                  archive_policy_name="low")
         self.index.create_resource('generic', r1, user, project,
@@ -378,8 +389,8 @@ class TestIndexerDriver(tests_base.TestCase):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1, user, project,
                                  archive_policy_name="low")
         self.index.create_metric(e2, user, project,
@@ -398,8 +409,8 @@ class TestIndexerDriver(tests_base.TestCase):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1, user, project,
                                  archive_policy_name="low")
         self.index.create_metric(e2, user, project,
@@ -416,8 +427,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_update_resource_attribute(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         rc = self.index.create_resource('instance', r1, user, project,
                                         flavor_id="1",
                                         image_ref="http://foo/bar",
@@ -429,8 +440,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_update_resource_no_change(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         rc = self.index.create_resource('instance', r1, user, project,
                                         flavor_id="1",
                                         image_ref="http://foo/bar",
@@ -447,8 +458,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_update_resource_ended_at_fail(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_resource('instance', r1, user, project,
                                    flavor_id="1",
                                    image_ref="http://foo/bar",
@@ -462,7 +473,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_update_resource_unknown_attribute(self):
         r1 = uuid.uuid4()
-        self.index.create_resource('instance', r1, uuid.uuid4(), uuid.uuid4(),
+        self.index.create_resource('instance', r1, str(uuid.uuid4()),
+                                   str(uuid.uuid4()),
                                    flavor_id="1",
                                    image_ref="http://foo/bar",
                                    host="foo",
@@ -475,7 +487,8 @@ class TestIndexerDriver(tests_base.TestCase):
     def test_update_non_existent_metric(self):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
-        self.index.create_resource('generic', r1, uuid.uuid4(), uuid.uuid4())
+        self.index.create_resource('generic', r1, str(uuid.uuid4()),
+                                   str(uuid.uuid4()))
         self.assertRaises(indexer.NoSuchMetric,
                           self.index.update_resource,
                           'generic',
@@ -484,7 +497,7 @@ class TestIndexerDriver(tests_base.TestCase):
     def test_update_non_existent_resource(self):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
-        self.index.create_metric(e1, uuid.uuid4(), uuid.uuid4(),
+        self.index.create_metric(e1, str(uuid.uuid4()), str(uuid.uuid4()),
                                  archive_policy_name="low")
         self.assertRaises(indexer.NoSuchResource,
                           self.index.update_resource,
@@ -497,15 +510,15 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertRaises(indexer.NoSuchMetric,
                           self.index.create_resource,
                           'generic',
-                          r1, uuid.uuid4(), uuid.uuid4(),
+                          r1, str(uuid.uuid4()), str(uuid.uuid4()),
                           metrics={'foo': e1})
 
     def test_delete_metric_on_resource(self):
         r1 = uuid.uuid4()
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1, user, project,
                                  archive_policy_name="low")
         self.index.create_metric(e2, user, project,
@@ -532,13 +545,15 @@ class TestIndexerDriver(tests_base.TestCase):
                           "created_by_project_id": project,
                           "user_id": None,
                           "project_id": None,
+                          "original_resource_id": None,
                           "type": "generic",
                           "metrics": {'bar': str(e2)}}, r.jsonify())
 
     def test_delete_instance(self):
         r1 = uuid.uuid4()
         created = self.index.create_resource('instance', r1,
-                                             uuid.uuid4(), uuid.uuid4(),
+                                             str(uuid.uuid4()),
+                                             str(uuid.uuid4()),
                                              flavor_id="123",
                                              image_ref="foo",
                                              host="dwq",
@@ -557,8 +572,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_list_resources_by_user(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         g = self.index.create_resource('generic', r1, user, project,
                                        user, project)
         resources = self.index.list_resources(
@@ -568,13 +583,13 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertEqual(g, resources[0])
         resources = self.index.list_resources(
             'generic',
-            attribute_filter={"=": {"user_id": uuid.uuid4()}})
+            attribute_filter={"=": {"user_id": 'bad-user'}})
         self.assertEqual(0, len(resources))
 
     def test_list_resources_by_created_by_user(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         g = self.index.create_resource('generic', r1, user, project)
         resources = self.index.list_resources(
             'generic',
@@ -583,13 +598,13 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertEqual(g, resources[0])
         resources = self.index.list_resources(
             'generic',
-            attribute_filter={"=": {"created_by_user_id": uuid.uuid4()}})
+            attribute_filter={"=": {"created_by_user_id": 'bad-user'}})
         self.assertEqual(0, len(resources))
 
     def test_list_resources_by_user_with_details(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         g = self.index.create_resource('generic', r1, user, project,
                                        user, project)
         r2 = uuid.uuid4()
@@ -611,8 +626,8 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_list_resources_by_project(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         g = self.index.create_resource('generic', r1, user, project,
                                        user, project)
         resources = self.index.list_resources(
@@ -622,13 +637,13 @@ class TestIndexerDriver(tests_base.TestCase):
         self.assertEqual(g, resources[0])
         resources = self.index.list_resources(
             'generic',
-            attribute_filter={"=": {"project_id": uuid.uuid4()}})
+            attribute_filter={"=": {"project_id": 'bad-project'}})
         self.assertEqual(0, len(resources))
 
     def test_list_resources_by_duration(self):
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         g = self.index.create_resource(
             'generic', r1, user, project,
             user_id=user, project_id=project,
@@ -656,10 +671,10 @@ class TestIndexerDriver(tests_base.TestCase):
         # for now it'll be better than nothing.
         r1 = uuid.uuid4()
         g = self.index.create_resource('generic', r1,
-                                       uuid.uuid4(), uuid.uuid4())
+                                       str(uuid.uuid4()), str(uuid.uuid4()))
         r2 = uuid.uuid4()
         i = self.index.create_resource('instance', r2,
-                                       uuid.uuid4(), uuid.uuid4(),
+                                       str(uuid.uuid4()), str(uuid.uuid4()),
                                        flavor_id="123",
                                        image_ref="foo",
                                        host="dwq",
@@ -698,6 +713,11 @@ class TestIndexerDriver(tests_base.TestCase):
             'generic',
             attribute_filter={"=": {"id": "f00bar" * 50}})
 
+    def test_list_resource_instance_flavor_id_numeric(self):
+        r = self.index.list_resources(
+            'instance', attribute_filter={"=": {"flavor_id": 1.0}})
+        self.assertEqual(0, len(r))
+
     def test_list_resource_weird_date(self):
         self.assertRaises(
             indexer.QueryValueError,
@@ -708,10 +728,10 @@ class TestIndexerDriver(tests_base.TestCase):
     def test_list_resources_without_history(self):
         e = uuid.uuid4()
         rid = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
-        new_user = uuid.uuid4()
-        new_project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
+        new_user = str(uuid.uuid4())
+        new_project = str(uuid.uuid4())
 
         self.index.create_metric(e, user, project,
                                  archive_policy_name="low")
@@ -737,10 +757,10 @@ class TestIndexerDriver(tests_base.TestCase):
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
         rid = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
-        new_user = uuid.uuid4()
-        new_project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
+        new_user = str(uuid.uuid4())
+        new_project = str(uuid.uuid4())
 
         self.index.create_metric(e1, user, project,
                                  archive_policy_name="low")
@@ -777,10 +797,10 @@ class TestIndexerDriver(tests_base.TestCase):
         e1 = uuid.uuid4()
         e2 = uuid.uuid4()
         rid = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
-        new_user = uuid.uuid4()
-        new_project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
+        new_user = str(uuid.uuid4())
+        new_project = str(uuid.uuid4())
 
         self.index.create_metric(e1, user, project,
                                  archive_policy_name="low")
@@ -824,8 +844,8 @@ class TestIndexerDriver(tests_base.TestCase):
         # database for all tests and the tests are running concurrently, but
         # for now it'll be better than nothing.
         r1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         g = self.index.create_resource(
             'generic', r1, user, project,
             started_at=utils.datetime_utc(2000, 1, 1, 23, 23, 23),
@@ -887,13 +907,13 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_get_metric(self):
         e1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1,
                                  user, project,
                                  archive_policy_name="low")
 
-        metric = self.index.get_metrics([e1])
+        metric = self.index.list_metrics(id=e1)
         self.assertEqual(1, len(metric))
         metric = metric[0]
         self.assertEqual(e1, metric.id)
@@ -904,13 +924,13 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_get_metric_with_details(self):
         e1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1,
                                  user, project,
                                  archive_policy_name="low")
 
-        metric = self.index.get_metrics([e1])
+        metric = self.index.list_metrics(id=e1)
         self.assertEqual(1, len(metric))
         metric = metric[0]
         self.assertEqual(e1, metric.id)
@@ -922,28 +942,35 @@ class TestIndexerDriver(tests_base.TestCase):
 
     def test_get_metric_with_bad_uuid(self):
         e1 = uuid.uuid4()
-        self.assertEqual([], self.index.get_metrics([e1]))
+        self.assertEqual([], self.index.list_metrics(id=e1))
 
     def test_get_metric_empty_list_uuids(self):
-        self.assertEqual([], self.index.get_metrics([]))
-
-    def test_get_metric_no_args(self):
-        self.assertRaises(TypeError, self.index.get_metrics, *[])
+        self.assertEqual([], self.index.list_metrics(ids=[]))
 
     def test_list_metrics(self):
         e1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1,
                                  user, project,
                                  archive_policy_name="low")
+        e2 = uuid.uuid4()
+        self.index.create_metric(e2,
+                                 user, project,
+                                 archive_policy_name="low")
         metrics = self.index.list_metrics()
-        self.assertIn(e1, [m.id for m in metrics])
+        id_list = [m.id for m in metrics]
+        self.assertIn(e1, id_list)
+        # Test ordering
+        if e1 < e2:
+            self.assertLess(id_list.index(e1), id_list.index(e2))
+        else:
+            self.assertLess(id_list.index(e2), id_list.index(e1))
 
     def test_list_metrics_delete_status(self):
         e1 = uuid.uuid4()
-        user = uuid.uuid4()
-        project = uuid.uuid4()
+        user = str(uuid.uuid4())
+        project = str(uuid.uuid4())
         self.index.create_metric(e1,
                                  user, project,
                                  archive_policy_name="low")
